#include "client/device.h"
#include "client/setup.h"

#include "tools/select.h"
#include "tools/string.h"

#include <vdr/channels.h>
#include <vdr/ringbuffer.h>
#include <vdr/eit.h>
#include <vdr/timers.h>

#include <time.h>
#include <iostream>

using namespace std;

#define VIDEOBUFSIZE MEGABYTE(3)

cStreamdevDevice *cStreamdevDevice::m_Device = NULL;

cStreamdevDevice::cStreamdevDevice(void) {
	m_Channel    = NULL;
	m_TSBuffer   = NULL;

#if VDRVERSNUM < 10300
#	if defined(HAVE_AUTOPID)
	(void)new cSIProcessor(new cSectionsScanner(""));
#	else
	(void)new cSIProcessor("");
# endif
	cSIProcessor::Read();
#else
	StartSectionHandler();
	cSchedules::Read();
#endif

	m_Device = this;

	if (StreamdevClientSetup.SyncEPG)	
		ClientSocket.SynchronizeEPG();
}

cStreamdevDevice::~cStreamdevDevice() {
	Dprintf("Device gets destructed\n");
	m_Device = NULL;
}

bool cStreamdevDevice::ProvidesSource(int Source) const {
	Dprintf("ProvidesSource, Source=%d\n", Source);
	return false;
}

bool cStreamdevDevice::ProvidesChannel(const cChannel *Channel, 
		int Priority, bool *NeedsDetachReceivers) const {
	Dprintf("ProvidesChannel, Channel=%s, Prio=%d\n", Channel->Name(), Priority);
	if (NeedsDetachReceivers)
		*NeedsDetachReceivers = false;
	return ClientSocket.ProvidesChannel(Channel, Priority);
}

bool cStreamdevDevice::SetChannelDevice(const cChannel *Channel, 
		bool LiveView) {
	Dprintf("SetChannelDevice Channel: %s, LiveView: %s\n", Channel->Name(),
			LiveView ? "true" : "false");

	if (LiveView)
		return false;

	if (ClientSocket.DataSocket(siLive) != NULL && Channel->Frequency() 
			== m_Channel->Frequency())
		return true;

	m_Channel = Channel;
	return ClientSocket.CreateDataConnection(siLive) 
			&& ClientSocket.SetChannelDevice(m_Channel);
}

bool cStreamdevDevice::SetPid(cPidHandle *Handle, int Type, bool On) {
	Dprintf("SetPid, Pid=%d, Type=%d, On=%d\n", Handle->pid, Type, On);
	return ClientSocket.SetPid(Handle->pid, On);
}

bool cStreamdevDevice::OpenDvr(void) {
	Dprintf("OpenDvr\n");
	m_TSBuffer = new cTSBuffer(*ClientSocket.DataSocket(siLive), MEGABYTE(2), 
			CardIndex() + 1);
	return true;
}

void cStreamdevDevice::CloseDvr(void) {
	Dprintf("CloseDvr\n");

	ClientSocket.CloseDvr();
	DELETENULL(m_TSBuffer);
}

bool cStreamdevDevice::GetTSPacket(uchar *&Data) {
  if (m_TSBuffer) {
    int r = m_TSBuffer->Read();
    if (r >= 0) {
      Data = m_TSBuffer->Get();
      return true;
    } else if (FATALERRNO) {
      LOG_ERROR;
    	return false;
    }
  	return true;
	}
  return false;
}

bool cStreamdevDevice::Init(void) {
	if (m_Device == NULL && StreamdevClientSetup.StartClient)
		new cStreamdevDevice;
	return true;
}

bool cStreamdevDevice::ReInit(void) {
	ClientSocket.Quit();
	ClientSocket.Reset();
	return StreamdevClientSetup.StartClient ? Init() : true;
}

