#include <vdr/menuitems.h>

#include "client/menu.h"
#include "client/player.h"
#include "client/socket.h"
#include "i18n.h"

#define CHNUMWIDTH  (numdigits(Channels.MaxNumber()) + 1)

// --- cMenuText -------------------------------------------------------------

class cMenuText : public cOsdMenu {
public:
  cMenuText(const char *Title, const char *Text, eDvbFont Font = fontOsd);
  virtual eOSState ProcessKey(eKeys Key);
  };

// --- cStreamdevMenu --------------------------------------------------------

cStreamdevMenu::cStreamdevMenu(void):
		cOsdMenu(tr("Streaming Control")) {
	SetHasHotkeys();
	Add(new cOsdItem(hk(tr("Remote Timers")),     (eOSState)subTimers));
	Add(new cOsdItem(hk(tr("Remote Recordings")), (eOSState)subRecordings));
	Add(new cOsdItem(hk(tr("Suspend Server")),    (eOSState)subSuspend));
	Add(new cOsdItem(hk(tr("Synchronize EPG")),   (eOSState)subSyncEPG));
}

cStreamdevMenu::~cStreamdevMenu() {
}

eOSState cStreamdevMenu::ProcessKey(eKeys Key) {
	eOSState state = cOsdMenu::ProcessKey(Key);
	switch (state) {
	case subTimers:     return AddSubMenu(new cStreamdevMenuTimers);
	case subRecordings: return AddSubMenu(new cStreamdevMenuRecordings);
	case subSuspend:    SuspendServer(); return osEnd;
	case subSyncEPG:		ClientSocket.SynchronizeEPG(); return osEnd;
	default:            return state;
	}
}

void cStreamdevMenu::SuspendServer(void) {
	cTBString buffer;

	if (ClientSocket.SuspendServer())
		Interface->Info(tr("Server is suspended"));
	else
		Interface->Error(tr("Couldn't suspend Server!"));
}

// --- cStreamdevMenuRecordingItem -------------------------------------------

class cStreamdevMenuRecordingItem: public cOsdItem {
private:
	int   m_Total;
	int   m_New;
	char *m_FileName;
	char *m_Name;

public:
	cStreamdevMenuRecordingItem(cRemoteRecording *Recording, int Level);
	virtual ~cStreamdevMenuRecordingItem();

	void IncrementCounter(bool New);
	const char *Name(void) const { return m_Name; }
	const char *FileName(void) const { return m_FileName; }
	bool IsDirectory(void) const { return m_Name != NULL; }
};

cStreamdevMenuRecordingItem::cStreamdevMenuRecordingItem(
		cRemoteRecording *Recording, int Level) {
	m_FileName = strdup(Recording->Name());
	m_Name = NULL;
	m_Total = m_New = 0;
	SetText(Recording->Title('\t', true, Level));
	if (*Text() == '\t')
		m_Name = strdup(Text() + 2);
}

cStreamdevMenuRecordingItem::~cStreamdevMenuRecordingItem() {
}

void cStreamdevMenuRecordingItem::IncrementCounter(bool New) {
	++m_Total;
	if (New) ++m_New;
	char *buffer = NULL;
	asprintf(&buffer, "%d\t%d\t%s", m_Total, m_New, m_Name);
	SetText(buffer, false);
}

// --- cStreamdevMenuRecordings ----------------------------------------------

cRemoteRecordings cStreamdevMenuRecordings::Recordings;
int cStreamdevMenuRecordings::HelpKeys = -1;

cStreamdevMenuRecordings::cStreamdevMenuRecordings(const char *Base, int Level,
		bool OpenSubMenus):
		cOsdMenu(Base ? Base : tr("Remote Recordings"), 6, 6) {
	m_Base = Base ? strdup(Base) : NULL;
	m_Level = Setup.RecordingDirs ? Level : -1;

  Display(); // this keeps the higher level menus from showing up briefly when 
	           // pressing 'Back' during replay

	if (!Base) {
		Interface->Status(tr("Fetching recordings..."));
		Interface->Flush();
	}

	if (Base || Recordings.Load()) {
		cStreamdevMenuRecordingItem *LastItem = NULL;
		char *LastItemText = NULL;
		for (int it = 0; it < (int)Recordings.Count(); ++it) {
			if (!Base || (strstr(Recordings[it]->Name(), Base) 
					== Recordings[it]->Name() && Recordings[it]->Name()[strlen(Base)] 
					== '~')) {
				cStreamdevMenuRecordingItem *Item 
						= new cStreamdevMenuRecordingItem(Recordings[it], m_Level);
				if (*Item->Text() && (!LastItem || strcmp(Item->Text(), LastItemText) 
						!= 0)) {
					Add(Item);
					LastItem = Item;
					free(LastItemText);
					LastItemText = strdup(LastItem->Text());
				} else
					delete Item;

				if (LastItem) {
					if (LastItem->IsDirectory())
						LastItem->IncrementCounter(Recordings[it]->IsNew());
				}
			}
		}
		free(LastItemText);
		if (Current() < 0)
			SetCurrent(First());
		else if (OpenSubMenus && Open(true))
			return;
	}
	SetHelpKeys();
}

cStreamdevMenuRecordings::~cStreamdevMenuRecordings() {
	if (m_Base != NULL) free(m_Base);
	// Save some memory
	Recordings.Clear();
	HelpKeys = -1;
}

void cStreamdevMenuRecordings::SetHelpKeys(void) {
  cStreamdevMenuRecordingItem *ri =(cStreamdevMenuRecordingItem*)Get(Current());
  int NewHelpKeys = HelpKeys;
  if (ri) {
     if (ri->IsDirectory())
        NewHelpKeys = 1;
     else {
        NewHelpKeys = 2;
        cRemoteRecording *recording = GetRecording(ri);
        if (recording && recording->Summary())
           NewHelpKeys = 3;
        }
     }
  if (NewHelpKeys != HelpKeys) {
     switch (NewHelpKeys) {
       case 0: SetHelp(NULL); break;
       case 1: SetHelp(NULL); break; //SetHelp(tr("Open")); break; XXX
       case 2:
       case 3: SetHelp(NULL, NULL, tr("Delete"), NewHelpKeys == 3 ? tr("Summary") : NULL);//SetHelp(tr("Play"), tr("Rewind"), tr("Delete"), NewHelpKeys == 3 ? tr("Summary") : NULL); XXX
       }
     HelpKeys = NewHelpKeys;
     }
}

cRemoteRecording *cStreamdevMenuRecordings::GetRecording(
		cStreamdevMenuRecordingItem *Item) {
  cRemoteRecording *recording = Recordings.GetByName(Item->FileName());
  if (!recording)
     Interface->Error(tr("Error while accessing recording!"));
  return recording;
}

bool cStreamdevMenuRecordings::Open(bool OpenSubMenus) {
	cStreamdevMenuRecordingItem *ri 
			= (cStreamdevMenuRecordingItem*)Get(Current());

	if (ri && ri->IsDirectory()) {
		const char *t = ri->Name();
		char *buffer = NULL;
		if (m_Base) {
			asprintf(&buffer, "%s~%s", m_Base, t);
			t = buffer;
		}
		AddSubMenu(new cStreamdevMenuRecordings(t, m_Level + 1, OpenSubMenus));
		if (buffer != NULL) free(buffer);
		return true;
	}
	return false;
}

eOSState cStreamdevMenuRecordings::Select(void) {	
	cStreamdevMenuRecordingItem *ri 
			= (cStreamdevMenuRecordingItem*)Get(Current());

	if (ri) {
		if (ri->IsDirectory())
			Open();
		/*else {
			cControl::Launch(new cStreamdevPlayerControl(ri->FileName()));
			return osEnd;
		} XXX */
	}
	return osContinue;
}

eOSState cStreamdevMenuRecordings::Delete(void) {
  if (HasSubMenu() || Count() == 0)
    return osContinue;
  cStreamdevMenuRecordingItem *ri 
			= (cStreamdevMenuRecordingItem*)Get(Current());
  if (ri && !ri->IsDirectory()) {
    if (Interface->Confirm(tr("Delete recording?"))) {
      cRemoteRecording *recording = GetRecording(ri);
      if (recording) {
        if (ClientSocket.DeleteRecording(recording)) {
          cOsdMenu::Del(Current());
          Recordings.Remove(recording);
          Display();
          if (!Count())
            return osBack;
        } else
          Interface->Error(tr("Error while deleting recording!"));
      }
    }
  }
  return osContinue;
}

eOSState cStreamdevMenuRecordings::Summary(void) {
  if (HasSubMenu() || Count() == 0)
    return osContinue;
  cStreamdevMenuRecordingItem *ri=(cStreamdevMenuRecordingItem *)Get(Current());
  if (ri && !ri->IsDirectory()) {
    cRemoteRecording *recording = GetRecording(ri);
    if (recording && recording->Summary() && *recording->Summary())
       return AddSubMenu(new cMenuText(tr("Summary"), recording->Summary()));
  }
  return osContinue;
}

eOSState cStreamdevMenuRecordings::ProcessKey(eKeys Key) {
	bool HadSubMenu = HasSubMenu();
	eOSState state = cOsdMenu::ProcessKey(Key);

	if (state == osUnknown) {
		switch (Key) {
		case kOk:     
		case kRed:    return Select();
		case kYellow: return Delete();
		case kBlue:   return Summary();
		default:      break;
		}
	}

	if (Key == kYellow && HadSubMenu && !HasSubMenu()) {
		cOsdMenu::Del(Current());
		if (!Count())
			return osBack;
		Display();
	}

	if (!HasSubMenu() && Key != kNone)
		SetHelpKeys();
	return state;
}

// --- cMenuEditChanItem -----------------------------------------------------

class cMenuEditChanItem : public cMenuEditIntItem {
protected:
  virtual void Set(void);
public:
  cMenuEditChanItem(const char *Name, int *Value);
  virtual eOSState ProcessKey(eKeys Key);
  };

// --- cMenuEditDateItem -----------------------------------------------------

class cMenuEditDateItem : public cMenuEditItem {
protected:
  time_t *value;
  virtual void Set(void);
public:
  cMenuEditDateItem(const char *Name, time_t *Value);
  virtual eOSState ProcessKey(eKeys Key);
  };

// --- cMenuEditDayItem ------------------------------------------------------

class cMenuEditDayItem : public cMenuEditIntItem {
protected:
  static int days[];
  int d;
  virtual void Set(void);
public:
  cMenuEditDayItem(const char *Name, int *Value);
  virtual eOSState ProcessKey(eKeys Key);
  };

// --- cMenuEditTimeItem -----------------------------------------------------

class cMenuEditTimeItem : public cMenuEditItem {
protected:
  int *value;
  int hh, mm;
  int pos;
  virtual void Set(void);
public:
  cMenuEditTimeItem(const char *Name, int *Value);
  virtual eOSState ProcessKey(eKeys Key);
  };

// --- cStreamdevMenuEditTimer -----------------------------------------------

class cStreamdevMenuEditTimer : public cOsdMenu {
private:
  int                m_Channel;
  bool               m_AddIfConfirmed;
  cRemoteTimer      *m_Timer;
  cRemoteTimer       m_Data;
  cRemoteTimers     *m_Timers;
  cMenuEditDateItem *m_FirstDay;

protected:
  void SetFirstDayItem(void);

public:
  cStreamdevMenuEditTimer(cRemoteTimers *Timers, cRemoteTimer *Timer, 
			bool New = false);
  virtual ~cStreamdevMenuEditTimer();

  virtual eOSState ProcessKey(eKeys Key);
};

cStreamdevMenuEditTimer::cStreamdevMenuEditTimer(cRemoteTimers *Timers,
		cRemoteTimer *Timer, bool New):
		cOsdMenu(tr("Edit remote timer"), 12) {
  m_FirstDay       = NULL;
  m_Timer          = Timer;
	m_Timers         = Timers;
  m_AddIfConfirmed = New;

  if (m_Timer) {
    m_Data = *m_Timer;
    if (New)
      m_Data.m_Active = 1;
    m_Channel = m_Data.Channel()->Number();
    Add(new cMenuEditBoolItem(tr("Active"),       &m_Data.m_Active));
    Add(new cMenuEditChanItem(tr("Channel"),      &m_Channel));
    Add(new cMenuEditDayItem( tr("Day"),          &m_Data.m_Day));
    Add(new cMenuEditTimeItem(tr("Start"),        &m_Data.m_Start));
    Add(new cMenuEditTimeItem(tr("Stop"),         &m_Data.m_Stop));
    Add(new cMenuEditIntItem( tr("Priority"),     &m_Data.m_Priority, 0, 
				MAXPRIORITY));
    Add(new cMenuEditIntItem( tr("Lifetime"),     &m_Data.m_Lifetime, 0, 
				MAXLIFETIME));
    Add(new cMenuEditStrItem( tr("File"),          m_Data.m_File, 
				sizeof(m_Data.m_File), tr(FileNameChars)));
    SetFirstDayItem();
	}
}

cStreamdevMenuEditTimer::~cStreamdevMenuEditTimer() {
  if (m_Timer && m_AddIfConfirmed) {
		Dprintf("SOMETHING GETS DELETED\n");
    delete m_Timer; // apparently it wasn't confirmed
	}
}

void cStreamdevMenuEditTimer::SetFirstDayItem(void) {
  if (!m_FirstDay && !m_Data.IsSingleEvent()) {
    Add(m_FirstDay = new cMenuEditDateItem(tr("First day"),&m_Data.m_FirstDay));
    Display();
  } else if (m_FirstDay && m_Data.IsSingleEvent()) {
    Del(m_FirstDay->Index());
    m_FirstDay = NULL;
    m_Data.m_FirstDay = 0;
    Display();
  }
}

eOSState cStreamdevMenuEditTimer::ProcessKey(eKeys Key) {
  eOSState state = cOsdMenu::ProcessKey(Key);

  if (state == osUnknown) {
    switch (Key) {
    case kOk:     
			{  
				cChannel *ch = Channels.GetByNumber(m_Channel);
				if (ch)
					m_Data.m_Channel = ch;
				else {
					Interface->Error(tr("*** Invalid Channel ***"));
					break;
				}
				if (!*m_Data.m_File)
					strcpy(m_Data.m_File, m_Data.Channel()->Name());
				if (m_Timer) {
					bool success = true;
					if (m_Data != *m_Timer) {
						// Timer has changed
						if ((success = ClientSocket.SaveTimer(m_Timer, m_Data))) {
							*m_Timer = m_Data;
							if (m_Timer->m_Active)
								m_Timer->m_Active = 1; 
								// allows external programs to mark active timers with 
								// values > 1 and recognize if the user has modified them
						}
					}
					if (success) {
						if (m_AddIfConfirmed)
							m_Timers->Append(m_Timer);
						isyslog("timer %d %s (%s)", m_Timer->Index() + 1, 
								m_AddIfConfirmed ? "added" : "modified", 
								m_Timer->m_Active ? "active" : "inactive");
						m_AddIfConfirmed = false;
					}
				}
			}
			return osBack;

		case kRed:
    case kGreen:
    case kYellow:
    case kBlue:   return osContinue;
    default: break;
		}
  }
  if (Key != kNone)
     SetFirstDayItem();
  return state;
}

// --- cStreamdevMenuTimerItem -----------------------------------------------

class cStreamdevMenuTimerItem: public cOsdItem {
private:
	cRemoteTimer *m_Timer;

public:
	cStreamdevMenuTimerItem(cRemoteTimer *Timer);
	virtual ~cStreamdevMenuTimerItem();

	virtual void Set(void);

	cRemoteTimer *Timer(void) const { return m_Timer; }
};

cStreamdevMenuTimerItem::cStreamdevMenuTimerItem(cRemoteTimer *Timer) {
	m_Timer = Timer;
	Set();
}

cStreamdevMenuTimerItem::~cStreamdevMenuTimerItem() {
}

void cStreamdevMenuTimerItem::Set(void) {
  char *buffer = NULL;
  asprintf(&buffer, "%c\t%d\t%s\t%02d:%02d\t%02d:%02d\t%s",
                    !m_Timer->Active() ? ' ' : 
										m_Timer->FirstDay() ? '!' : 
										/*m_Timer->Recording() ? '#' :*/ '>',
                    m_Timer->Channel()->Number(),
                    m_Timer->PrintDay(m_Timer->Day()),
                    m_Timer->Start() / 100,
                    m_Timer->Start() % 100,
                    m_Timer->Stop() / 100,
                    m_Timer->Stop() % 100,
                    m_Timer->File());
  SetText(buffer, false);
}

// --- cStreamdevMenuTimers --------------------------------------------------

cRemoteTimers cStreamdevMenuTimers::Timers;

cStreamdevMenuTimers::cStreamdevMenuTimers(void):
		cOsdMenu(tr("Remote Timers"), 2, CHNUMWIDTH, 10, 6, 6) {
	Refresh();
  SetHelp(tr("Edit"), tr("New"), tr("Delete"), tr("On/Off"));
}

cStreamdevMenuTimers::~cStreamdevMenuTimers() {
	// Save some memory
	Timers.Clear();
}

eOSState cStreamdevMenuTimers::ProcessKey(eKeys Key) {
	int timerNum = HasSubMenu() ? Count() : -1;
	eOSState state = cOsdMenu::ProcessKey(Key);

	if (state == osUnknown) {
		switch (Key) {
		case kOk:     return Summary();
		case kRed:    return Edit();
		case kGreen:  return New();
		case kYellow: return Delete();
		case kBlue:   OnOff(); break;
		default:      break;
		}
	}

	if (timerNum >= 0 && !HasSubMenu()) {
		Refresh();
		Display();
	}
	return state;
}

eOSState cStreamdevMenuTimers::Edit(void) {
  if (HasSubMenu() || Count() == 0)
    return osContinue;
  isyslog("Streamdev: Editing remote timer %d", CurrentTimer()->Index() + 1);
  return AddSubMenu(new cStreamdevMenuEditTimer(&Timers, CurrentTimer()));
}

eOSState cStreamdevMenuTimers::New(void) {
  if (HasSubMenu())
    return osContinue;
  return AddSubMenu(new cStreamdevMenuEditTimer(&Timers, new cRemoteTimer, 
			true));
}

eOSState cStreamdevMenuTimers::Delete(void) {
	cRemoteTimer *ti = CurrentTimer();
	if (ti) {
		if (Interface->Confirm(tr("Delete timer?"))) {
			int idx = ti->Index();
			if (ClientSocket.DeleteTimer(ti)) {
				Timers.Remove(ti);
				cOsdMenu::Del(Current());
				Display();
				isyslog("Streamdev: Remote timer %d deleted", idx + 1);
			} else {
				Refresh();
				Display();
			}
		}
	}
	return osContinue;
}

eOSState cStreamdevMenuTimers::OnOff(void) {
	cRemoteTimer *timer = CurrentTimer();
	if (timer) {
		cRemoteTimer data = *timer;
		data.OnOff();
		if (data.FirstDay())
			isyslog("Streamdev: Remote timer %d first day set to %s", 
					data.Index() + 1, data.PrintFirstDay());
    else
      isyslog("Streamdev: Remote timer %d %sactivated", data.Index() + 1, 
					data.Active() ? "" : "de");

		if (ClientSocket.SaveTimer(timer, data)) {
			*timer = data;
			RefreshCurrent();
			DisplayCurrent(true);
		} else {
			Refresh();
			Display();
		}
	}
	return osContinue;
}

eOSState cStreamdevMenuTimers::Summary(void) {
	if (HasSubMenu() || Count() == 0)
		return osContinue;

	cRemoteTimer *ti = CurrentTimer();
	if (ti && !ti->Summary().IsNull())
		return AddSubMenu(new cMenuText(tr("Summary"), ti->Summary()));

	return osContinue;
}

cRemoteTimer *cStreamdevMenuTimers::CurrentTimer(void) {
	cStreamdevMenuTimerItem *item = (cStreamdevMenuTimerItem*)Get(Current());
	return item ? item->Timer() : NULL;
}

void cStreamdevMenuTimers::Refresh(void) {
	Clear();
	if (Timers.Load()) {
		for (int it = 0; it < (int)Timers.Count(); ++it) {
			Add(new cStreamdevMenuTimerItem(Timers[it]));
		}
	}
}
