#include "client/pes2ts.h"
#include "common.h"

#include <vdr/ringbuffer.h>
#include <tools/select.h>
#include <tools/socket.h>

int cPES2TSRemux::m_TSPutFd = -1;

void put_tsfile(uint8_t *Data, int Length, void *P2P) {
	::write(cPES2TSRemux::m_TSPutFd, Data, Length);
}

cPES2TSRemux::cPES2TSRemux(cTBSocket *File, uint Apid, uint Vpid)
#if VDRVERSNUM >= 10300
		: cThread("Streamdev: pes2ts remux")
#endif
{
	m_Active = false;
	m_PESFile = File;
	m_TSFile = NULL;
	m_Files[0] = -1;
	m_Files[1] = -1;
	m_Apid = Apid;
	m_Vpid = Vpid;
	
	if (::pipe(m_Files) == -1) {
		esyslog("Streamdev: Couldn't create pipe: %s", strerror(errno));
		return;
	}
	m_TSPutFd = m_Files[1];
	::fcntl(m_Files[0], F_SETFL, O_NONBLOCK);

	m_TSFile = new cTBSocket;
	m_TSFile->Open(m_Files[0]);

	Start();
}

cPES2TSRemux::~cPES2TSRemux(void) {
	Stop();
	DELETENULL(m_TSFile);
}

void cPES2TSRemux::Stop(void) {
	Dprintf("Remuxer gets stopped\n");
	close(m_Files[1]);
	m_Files[0] = -1;
	m_Files[1] = -1;
	m_Active = false;
	Cancel(3);
}

void cPES2TSRemux::Action(void) {
	cTBSelect select;
	p2p P2P;
	uchar pesdata[BUFSIZ];

#if VDRVERSNUM < 10300
	isyslog("Streamdev: Remux thread started (pid = %d)", getpid());
#endif

	m_Active = true;
	init_p2p(&P2P, NULL, 2048);
	P2P.pida = m_Apid;
	P2P.pidv = m_Vpid;
	P2P.acounter = 0;
	P2P.vcounter = 0;
	P2P.count1 = 0;
	P2P.count0 = 0;
	P2P.func = put_tsfile;

	while (m_Active) {	
		int bytes;

		select.Clear();
		select.Add(*m_PESFile, false);

		if (select.Select() == -1 || (bytes = m_PESFile->Read(pesdata, 
				sizeof(pesdata))) == -1) {
			esyslog("Streamdev: Couldn't read PES data: %s", strerror(errno));
			m_Active = false;
			continue;
		}

		get_pes((uint8_t*)pesdata, bytes, &P2P, pes_in_ts);
	}

#if VDRVERSNUM < 10300
	isyslog("Streamdev: Remux thread stopped");
#endif
}

