#include <vdr/ringbuffer.h>
#include <vdr/remux.h>

#include "client/player.h"
#include "client/socket.h"
#include "common.h"

#define VIDEOBUFSIZE MEGABYTE(3)

cStreamdevPlayer::cStreamdevPlayer(const char *Filename)
#if VDRVERSNUM >= 10300
		:cThread("Streamdev: player")
#endif
{
	m_Active      = false;
	m_FirstPacket = true;
	m_Socket      = NULL;
	m_ReadFrame   = NULL;
	m_PlayFrame   = NULL;
	m_RingBuffer  = new cRingBufferFrame(VIDEOBUFSIZE);

	Dprintf("Playing: %s\n", Filename);
	ClientSocket.CreateDataConnection(siReplay) 
			&& ClientSocket.StartReplay(Filename);

	m_Socket = ClientSocket.DataSocket(siReplay);
}

cStreamdevPlayer::~cStreamdevPlayer() {
	ClientSocket.AbortReplay();
	Detach();
	delete m_RingBuffer;
}

void cStreamdevPlayer::Activate(bool On) {
	if (On)
		Start();
	else if (m_Active) {
		m_Active = false;
		Cancel(3);
	}
}

void cStreamdevPlayer::Action(void) {
	uchar *p = NULL;
	int pc = 0;

#if VDRVERSNUM < 10300
	isyslog("Streamdev: player thread started (pid=%d)", getpid());
#endif

	m_Active = true;
	while (m_Active) {
		cPoller Poller;
		if (DevicePoll(Poller, 100)) {
			LOCK_THREAD;

			// Read next frame:
			if (!m_ReadFrame) {
				uchar buf[MAXFRAMESIZE];
				int r = m_Socket->Read(buf, MAXFRAMESIZE);
				if (r > 0)
					m_ReadFrame = new cFrame(buf, r, ftUnknown);
				else if (r == 0) {
					m_Active = false;
					break;
				} else if (r < 0 && FATALERRNO) {
					LOG_ERROR;
					m_Active = false;
					break;
				}
			}

			// Store frame in buffer:
			if (m_ReadFrame) {
				if (m_RingBuffer->Put(m_ReadFrame))
					m_ReadFrame = NULL;
			}

			// Get frame from buffer:
			if (!m_PlayFrame) {
				m_PlayFrame = m_RingBuffer->Get();
			}

			// Play the frame:
			if (m_PlayFrame) {
				if (!p) {
					p = m_PlayFrame->Data();
					pc = m_PlayFrame->Count();
					if (p) {
						if (m_FirstPacket) {
							cRemux::SetBrokenLink(p, pc);
							m_FirstPacket = false;
						}
					}
				}

				if (p) {
					int w = PlayVideo(p, pc);
					if (w > 0) {
						p += w;
						pc -= w;
					} else if (w < 0 && FATALERRNO) {
						LOG_ERROR;
						break;
					}
				}

				if (pc == 0) {
					m_RingBuffer->Drop(m_PlayFrame);
					m_PlayFrame = NULL;
					p = NULL;
				}
			}
		}
	}

#if VDRVERSNUM < 10300
	isyslog("Streamdev: player thread stopped");
#endif
}

cStreamdevPlayerControl::cStreamdevPlayerControl(const char *Filename):
		cControl(m_Player = new cStreamdevPlayer(Filename)) {
	m_Visible = false;
}

cStreamdevPlayerControl::~cStreamdevPlayerControl() {
	Hide();
	DELETENULL(m_Player);
}

void cStreamdevPlayerControl::Hide() {
	if (m_Visible) {
		Interface->Close();
		m_Visible = false;
	}
}

eOSState cStreamdevPlayerControl::ProcessKey(eKeys Key) {
	if (!Active())
		return osEnd;

	switch (Key) {
	case kBlue:
	case kBack:
		Hide();
		DELETENULL(m_Player);
		return osEnd;

	default:
		return osUnknown;
	}
	return osContinue;
}
