#ifndef VDR_STREAMDEV_COMMON_H
#define VDR_STREAMDEV_COMMON_H

#include <vdr/tools.h>
#include <vdr/plugin.h>

#include "tools/socket.h"

#ifdef DEBUG
#	include <stdio.h>
#	define Dprintf(x...) fprintf(stderr, x)
#else
#	define Dprintf(x...)
#endif

class cChannel;

char *GetNextLine(char *String, uint Length, uint &Offset);

cChannel *ChannelFromString(char *String);

/* Disable logging if BUFCOUNT buffer overflows occur within BUFOVERTIME
   milliseconds. Enable logging again if there is no error within BUFOVERTIME
   milliseconds. */
#define BUFOVERTIME  5000
#define BUFOVERCOUNT 100

#define POLLFAIL esyslog("Streamdev: Polling failed: %s", strerror(errno))
#define WRITEFAIL esyslog("Streamdev: Writing failed: %s", strerror(errno))
#define READFAIL esyslog("Streamdev: Reading failed: %s", strerror(errno))
#define CHECKPOLL(x) if ((x)<0){POLLFAIL; return false;}
#define CHECKWRITE(x) if ((x)<0) { WRITEFAIL; return false; }
#define CHECKREAD(x) if ((x)<0) { READFAIL; return false; }

enum eStreamType {
	stTS,
	stPES,
	st_ClientCount,
	stPS = st_ClientCount,
	stES,
	st_Count
};

enum eSuspendMode {
	smOffer,
	smAlways,
	smNever,
	sm_Count
};
	
enum eSocketId {
	siLive,
	siReplay,
	si_Count
};

extern const char *VERSION;
extern const char *StreamTypes[st_Count];
extern const char *SuspendModes[sm_Count];
extern const char  IpCharacters[];

class cStreamdevMenuSetupPage: public cMenuSetupPage {
protected:
	void AddCategory(const char *Title);
	virtual void Store(void) = 0;

	void AddBoolEdit(const char *Title, int &Value);
	void AddIpEdit(const char *Title, char *Value);
	void AddShortEdit(const char *Title, int &Value);
	void AddRangeEdit(const char *Title, int &Value, int Min, int Max);
	void AddSuspEdit(const char *Title, int &Value);
	void AddTypeEdit(const char *Title, int &Value, bool ClientOnly = false);
};

class cMenuEditIpItem: public cMenuEditItem {
private:
	char *value;
	int curNum;
	int pos;
	bool step;

protected:
	virtual void Set(void);

public:
	cMenuEditIpItem(const char *Name, char *Value); // Value must be 16 bytes
	~cMenuEditIpItem();

	virtual eOSState ProcessKey(eKeys Key);
};

#endif // VDR_STREAMDEV_COMMON_H
