#include "server/component.h"
#include "server/connection.h"

#include <vdr/tools.h>
#include <string.h>
#include <errno.h>

cServerComponent::cServerComponent(const char *Protocol, uint ListenPort) {
	m_Protocol = Protocol;
	m_ListenPort = ListenPort;
}

cServerComponent::~cServerComponent() {
}

bool cServerComponent::Init(void) {
	if (!m_Listen.Listen(m_ListenPort, 5)) {
		esyslog("Streamdev: Couldn't listen (%s) localhost:%d: %s", m_Protocol,
				m_ListenPort, strerror(errno));
		return false;
	}
	isyslog("Streamdev: Listening (%s) on port %d", m_Protocol, m_ListenPort);
	return true;
}

void cServerComponent::Exit(void) {
	m_Listen.Close();
}

cServerConnection *cServerComponent::CanAct(const cTBSelect &Select) {
	if (Select.CanRead(m_Listen)) {
		cServerConnection *client = NewConnection();
		if (client->Accept(m_Listen)) {
			isyslog("Streamdev: Accepted new client (%s) %s:%d", m_Protocol,
					(const char*)client->RemoteIp(), client->RemotePort());
			return client;
		} else {
			esyslog("Streamdev: Couldn't accept (%s): %s", m_Protocol, 
					strerror(errno));
			delete client;
		}
	}
	return NULL;
}
