#include "server/connection.h"
#include "server/setup.h"
#include "server/suspend.h"
#include "common.h"

#include <vdr/tools.h>
#include <string.h>
#include <errno.h>

cServerConnection::cServerConnection(const char *Protocol) {
	m_RdBytes = 0;
	m_WrBytes = 0;
	m_WrOffs = 0;
	m_DeferClose = false;
	m_Protocol = Protocol;
}

cServerConnection::~cServerConnection() {
}

bool cServerConnection::CanAct(const cTBSelect &Select) {
	if (Select.CanRead(*this)) {
		int b;
		if ((b = Read(m_RdBuf + m_RdBytes, sizeof(m_RdBuf) - m_RdBytes - 1)) < 0) {
			esyslog("Streamdev: Read from client (%s) %s:%d failed: %s", m_Protocol,
					(const char*)RemoteIp(), RemotePort(), strerror(errno));
			return false;
		}

		if (b == 0) {
			isyslog("Streamdev: Client (%s) %s:%d closed connection", m_Protocol,
					(const char*)RemoteIp(), RemotePort());
			return false;
		}

		m_RdBytes += b;
		m_RdBuf[m_RdBytes] = '\0';
		return ParseBuffer();
	}

	if (Select.CanWrite(*this)) {
		int b;
		if ((b = Write(m_WrBuf + m_WrOffs, m_WrBytes - m_WrOffs)) < 0) {
			esyslog("Streamdev: Write to client (%s) %s:%d failed: %s", m_Protocol,
					(const char*)RemoteIp(), RemotePort(), strerror(errno));
			return false;
		}

		m_WrOffs += b;
		if (m_WrOffs == m_WrBytes) {
			m_WrBytes = 0;
			m_WrOffs = 0;
		}
	}

	if (m_WrBytes == 0) {
		if (m_DeferClose)
			return false;
	}
	return true;
}

bool cServerConnection::ParseBuffer(void) {
	char *ep;
	bool res;

	while ((ep = strchr(m_RdBuf, '\012')) != NULL) {
		*ep = '\0';
		if (ep > m_RdBuf && *(ep-1) == '\015')
			*(ep-1) = '\0';

		Dprintf("IN: |%s|\n", m_RdBuf);
		res = Command(m_RdBuf);
		m_RdBytes -= ++ep - m_RdBuf;
		if (m_RdBytes > 0)
			memmove(m_RdBuf, ep, m_RdBytes);
		if (res == false)
			return false;
	}
	return true;
}

bool cServerConnection::Respond(const char *Message) {
	uint len = strlen(Message);
	if (m_WrBytes + len + 2 > sizeof(m_WrBuf)) {
		esyslog("Streamdev: Output buffer overflow (%s) for %s:%d", m_Protocol,
				(const char*)RemoteIp(), RemotePort());
		return false;
	}
	Dprintf("OUT: |%s|\n", Message);
	memcpy(m_WrBuf + m_WrBytes, Message, len);
	m_WrBuf[m_WrBytes + len] = '\015';
	m_WrBuf[m_WrBytes + len + 1] = '\012';
	m_WrBytes += len + 2;
	return true;
}
	
cDevice *cServerConnection::GetDevice(const cChannel *Channel, int Priority) {
	cDevice *device = NULL;
	int i;

	Dprintf("+ Statistics:\n");
	Dprintf("+ Current Channel: %d\n", cDevice::CurrentChannel());
	Dprintf("+ Current Device: %d\n", cDevice::ActualDevice()->CardIndex());
	Dprintf("+ Transfer Mode: %s\n", cDevice::ActualDevice() 
			== cDevice::PrimaryDevice() ? "false" : "true");
	Dprintf("+ Replaying: %s\n", cDevice::PrimaryDevice()->Replaying() ? "true"
			: "false");

	isyslog("Streamdev: Detaching streamer for device selection");
	DetachAll();

	for (i = 0; i < cDevice::NumDevices(); ++i) {
		Dprintf("Checking device %d\n", i);
		device = cDevice::GetDevice(i);
		if (device == cDevice::PrimaryDevice())
			// Prefer secondary devices over Primary
			continue;

		if (!device->ProvidesChannel(Channel, Priority))
			continue;
		
		Dprintf("Device %d claims it can provide this channel\n", i);
		if (device != cDevice::ActualDevice())
			// Should be ok if the current channel doesn't run there
			break;

		Dprintf("Device %d is ActualDevice\n", i);
		cChannel *current 
				= Channels.GetByNumber(cDevice::PrimaryDevice()->CurrentChannel());
		if (StreamdevServerSetup.SuspendMode == smAlways || cSuspendCtl::IsActive()
				|| current->Frequency() == Channel->Frequency())
			// If Live-TV is (always) suspended, we can use this one
			// Should be ok otherwise as long as the tuned Frequency is equal to 
			// the requested one
			break;
	}

	if (i == cDevice::NumDevices()) {
		Dprintf("Checking PrimaryDevice\n");
		device = cDevice::PrimaryDevice();
		if (device == cDevice::ActualDevice()) {
			Dprintf("PrimaryDevice is ActualDevice\n");
			cChannel *current 
					= Channels.GetByNumber(cDevice::PrimaryDevice()->CurrentChannel());
			if (current->Frequency() != Channel->Frequency() 
					&& !(cSuspendCtl::IsActive() || StreamdevServerSetup.SuspendMode 
					== smAlways))
				device = NULL;
		}
	}

	isyslog("Streamdev: Reattaching transceiver after device selection");
	AttachAll();

	return device;
}
