#include <vdr/recording.h>
#include <vdr/ringbuffer.h>
#include <vdr/videodir.h>

#include "server/filestreamer.h"
#include "common.h"

cStreamdevFileReceiver::cStreamdevFileReceiver(cRecording *Recording,
		cStreamdevFileStreamer *Streamer)
#if VDRVERSNUM >= 10300
		:cThread("Streamdev: File receiver")
#endif
{
	m_Active   = false;
	m_Eof      = false;
	m_FileName = new cFileName((cTBString)Recording->FileName(), false);
	m_Streamer = Streamer;

	Dprintf("opening: %s\n", m_FileName->Name());

	if (!m_ReplayFile.Open((uint)m_FileName->Open())) {
		esyslog("ERROR: Streamdev: Couldn't open recording \"%s\"", 
				m_FileName->Name());
		return;
	}

	Start();
}

cStreamdevFileReceiver::~cStreamdevFileReceiver() {
	Dprintf("Stopping and Killing File Reader\n");
	if (m_Active) {
		m_Active = false;
		Cancel(3);
	}

	delete m_FileName;
}

bool cStreamdevFileReceiver::NextFile(uchar FileNumber, int FileOffset) {
	if (FileNumber > 0)
		m_ReplayFile.Open((uint)m_FileName->SetOffset(FileNumber, FileOffset));
	else if (m_ReplayFile.IsOpen() && m_Eof)
		m_ReplayFile.Open((uint)m_FileName->NextFile());
	m_Eof = false;
	return m_ReplayFile.IsOpen();
}

void cStreamdevFileReceiver::Action(void) {
#if VDRVERSNUM < 10300
	isyslog("Streamdev: file receiver thread started (pid=%d)", getpid());
#endif

	m_Active = true;
	while (m_Active && NextFile()) {
		uchar buf[BUFSIZ];
		int r = m_ReplayFile.Read(buf, BUFSIZ);
		if (r > 0) {
			int len = m_Streamer->Put(buf, r);
			if (len != r) {	// Ringbuffer did WaitForPut (hopefully...)
				int rest = m_Streamer->Put(buf + len, r - len);
				len += rest;
				if (len != r)
					esyslog("ERROR: Streamdev: Ring buffer overflow (%d bytes dropped)",
							len - r);
			}
		} else if (r == 0 && errno == 0) {
			Dprintf("file has eorf\n");
			m_Eof = true;
		} else {
			LOG_ERROR;
			m_Active = false;
			break;
		}
	}
	m_Active = false;

#if VDRVERSNUM < 10300
	isyslog("Streamdev: file receiver thread stopped");
#endif
}

cStreamdevFileStreamer::cStreamdevFileStreamer(cTBSocket *Socket):
		cStreamdevStreamer("File streamer", Socket) {
	m_Receiver = NULL;
}

cStreamdevFileStreamer::~cStreamdevFileStreamer() {
	Dprintf("Killing file streamer and its reader\n");
	if (m_Receiver != NULL) delete m_Receiver;
}

bool cStreamdevFileStreamer::SetFile(cRecording *Recording) {
	m_Receiver = new cStreamdevFileReceiver(Recording, this);
	Start();
	return true;
}

